#
# script to aggregate the records/model data into grid-cells and to alculate the mean forest biome trend over these grid-cells, area-weighted, with uncertainties (calculated via bootstrapping)
#input: table of biomes in the model and in the reconstructions "25krun_rec_mod_datatNH30.csv", provided in this archive.
# needs also a list of problematic records, which are excluded ("prob_records_0kRF.csv")
# plots Figure 2, with 4 sub-figures (NH, Asia, Europe, North America)

######################################
--------- Preprocessing -------------
library(dplyr)

# read in data
total1 <- read.table("25krun_rec_mod_datatNH30.csv", header=FALSE)
colnames(total1) <- c("Longitude","Latitude","KaBP","biomREC","Mega_biome")
# Biomnames: 1-4: forest, grasland, desert, tundra
probrecs <- read.table("prob_records_0kRF.csv", header=FALSE)
colnames(probrecs) <- c("Longitude","Latitude","difference")
total <- anti_join(total1, probrecs, by = c("Longitude","Latitude"))
total$Mega_biome[total$Mega_biome > 1.5] <- 0
total$biomREC[total$biomREC > 1.5] <- 0
total<-na.omit(total)
total <- total[total$KaBP <= 22,]


#### regions
glob_lon_limits <- c(-180,180)
glob_lat_limits <- c(30,80)
euras_lon_limits <- c(-10,18)
euras_lat_limits <- c(30,80)
euro_lon_limits <- c(-10,50)
euro_lat_limits <- c(30,80)
nam_lon_limits <- c(-180,-47)
nam_lat_limits <- c(30,80)
asia_lon_limits <- c(50,180)
asia_lat_limits <- c(30,80)
easm_lon_limits <- c(100,120)
easm_lat_limits <- c(30,50)
sib_lon_limits <- c(50,180)
sib_lat_limits <- c(50,90)

resample <- function(x, ...) x[sample.int(length(x), ...)]


#Calculate spatial mean
spatial_means <- function(lon,lat,clim_field) {
  spatial_weights <- array(0,dim=c(length(lon),length(lat)))
  for (i in 1:length(lat)) {spatial_weights[,i] <- (cos((lat[i]+mean(diff(lat))/2)*pi/180)+cos((lat[i]-mean(diff(lat))/2)*pi/180))/2}
  spatial_weights[which(is.na(clim_field[1:(length(lon)*length(lat))]))] <- NA
  spatial_weights <- spatial_weights/sum(spatial_weights,na.rm=T)
  return(sum(spatial_weights*clim_field,na.rm=T))
}

# --------- Updated bootstrapping + lat-weighting 10x5 grid ----------------
lon_seq <- seq(-180,180,by=10)
lat_seq <- seq(30,80,by=5)
time <- sort(unique(total$KaBP))
nr_samples <- 1000
lon_limits <- glob_lon_limits #easm_lon_limits #euro_lon_limits #asia_lon_limits #nam_lon_limits
lat_limits <- glob_lat_limits #easm_lat_limits #euro_lat_limits #asia_lat_limits #nam_lat_limits

forest_frac_gc_rec <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_sim <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_rec_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_sim_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_rec_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_sim_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
nrec <- forest_frac_gc_rec_mean <- forest_frac_gc_sim_mean <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1))

for (t in 1:length(time)) {
  ind_tmp <- which(total$KaBP == time[t] & total$Longitude>=lon_limits[1] & total$Longitude<=lon_limits[2] & total$Latitude>=lat_limits[1] & total$Latitude<=lat_limits[2])
  rec <- total[ind_tmp,]
  for (k in 1:nr_samples) {
    ind_resample <- resample(1:nrow(rec),size=nrow(rec),replace=TRUE)
    rec_resample <- rec[ind_resample,]
    for (i in 1:(length(lon_seq)-1)) {
      for (j in 1:(length(lat_seq)-1)) {
        ind_tmp_local <- which(rec_resample$Longitude>=lon_seq[i] & rec_resample$Longitude<lon_seq[i+1] & rec_resample$Latitude>=lat_seq[j] & rec_resample$Latitude<lat_seq[j+1])
        if (length(ind_tmp_local)>=1) {
          forest_frac_gc_rec[t,i,j,k] <- mean(rec_resample$biomREC[ind_tmp_local])
          forest_frac_gc_sim[t,i,j,k] <- mean(rec_resample$Mega_biome[ind_tmp_local])
        }
      }
    }  
    forest_frac_gc_rec_agg[t,k] <- mean(forest_frac_gc_rec[t,,,k],na.rm=TRUE)
    forest_frac_gc_sim_agg[t,k] <- mean(forest_frac_gc_sim[t,,,k],na.rm=TRUE)
    forest_frac_gc_rec_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_rec[t,,,k])
    forest_frac_gc_sim_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_sim[t,,,k])
  }
}



# plot the figure, NH mean
pdf("gcmean_forest_fraction_uncertainty_10x5_NH_post.pdf")
par(mar = c(15, 4, 4, 4))

plot(-time*1000,apply(forest_frac_gc_rec_agg,1,mean),type="o",pch=20,col="white",ylim=c(0,1),yaxt="n",ylab="forest fraction", xaxt="n", xlab="years BP",main="NH mean forest fraction")
ci_sim <- t(apply(forest_frac_gc_sim_agg,1,quantile,c(0.05,0.95)))
ci_rec <- t(apply(forest_frac_gc_rec_agg,1,quantile,c(0.05,0.95)))

ytick<-seq(0.0, 1.0, by=0.2)
xtick<-seq(-25000,0,5000)
axis(side=2, at=ytick, las=1, cex.axis=1.3)
axis(side=1, at=xtick, las=1, cex.axis=1.3)

abline(h=seq(0,1,0.2), lty=3, col="darkgrey")
abline(v=seq(-25000,0,5000), lty=3, col="darkgrey")

#rect(-23000,-30,-19000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
#rect(-14690,-30,-12890,60, col= rgb(0.7,0,0,alpha=0.1), border = "transparent")
#rect(-12900,-30,-11600,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")

rect(-23000,-30,-18000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
rect(-10000,-30,0,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")


polygon(-c(time*1000,rev(time*1000)),c(ci_sim[,1],rev(ci_sim[,2])),col=adjustcolor("black",0.3),border="transparent")
polygon(-c(time*1000,rev(time*1000)),c(ci_rec[,1],rev(ci_rec[,2])),col=adjustcolor("red",0.3),border="transparent")
lines(-time*1000,apply(forest_frac_gc_rec_agg,1,mean),type="o",pch=20,col="red")
lines(-time*1000,apply(forest_frac_gc_sim_agg,1,mean),type="o",pch=20)
legend("bottomright", inset=.015, legend=c("MPI-ESM", "Records"), col=c("black", "red"), lwd=3, lty=c(1,1),  xpd=NA, bg='white',y.intersp=0.92,x.intersp=0.92)

dev.off()
############################
# Europe
############################################################
lon_seq <- seq(-180,180,by=10)
lat_seq <- seq(30,80,by=5)
time <- sort(unique(total$KaBP))
nr_samples <- 1000
lon_limits <- euro_lon_limits #easm_lon_limits #euro_lon_limits #asia_lon_limits #nam_lon_limits
lat_limits <- euro_lat_limits #easm_lat_limits #euro_lat_limits #asia_lat_limits #nam_lat_limits

forest_frac_gc_eur_rec <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_eur_sim <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_eur_rec_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_eur_sim_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_eur_rec_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_eur_sim_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
nrec <- forest_frac_gc_eur_rec_mean <- forest_frac_gc_eur_sim_mean <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1))

for (t in 1:length(time)) {
  ind_tmp <- which(total$KaBP == time[t] & total$Longitude>=lon_limits[1] & total$Longitude<=lon_limits[2] & total$Latitude>=lat_limits[1] & total$Latitude<=lat_limits[2])
  rec <- total[ind_tmp,]
  for (k in 1:nr_samples) {
    ind_resample <- resample(1:nrow(rec),size=nrow(rec),replace=TRUE)
    rec_resample <- rec[ind_resample,]
    for (i in 1:(length(lon_seq)-1)) {
      for (j in 1:(length(lat_seq)-1)) {
        ind_tmp_local <- which(rec_resample$Longitude>=lon_seq[i] & rec_resample$Longitude<lon_seq[i+1] & rec_resample$Latitude>=lat_seq[j] & rec_resample$Latitude<lat_seq[j+1])
        if (length(ind_tmp_local)>=1) {
          forest_frac_gc_eur_rec[t,i,j,k] <- mean(rec_resample$biomREC[ind_tmp_local])
          forest_frac_gc_eur_sim[t,i,j,k] <- mean(rec_resample$Mega_biome[ind_tmp_local])
        }
      }
    }  
    forest_frac_gc_eur_rec_agg[t,k] <- mean(forest_frac_gc_eur_rec[t,,,k],na.rm=TRUE)
    forest_frac_gc_eur_sim_agg[t,k] <- mean(forest_frac_gc_eur_sim[t,,,k],na.rm=TRUE)
    forest_frac_gc_eur_rec_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_eur_rec[t,,,k])
    forest_frac_gc_eur_sim_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_eur_sim[t,,,k])
  }
}
############################ NAmerica
lon_seq <- seq(-180,180,by=10)
lat_seq <- seq(30,80,by=5)
time <- sort(unique(total$KaBP))
nr_samples <- 1000
lon_limits <- nam_lon_limits #easm_lon_limits #euro_lon_limits #asia_lon_limits #nam_lon_limits
lat_limits <- nam_lat_limits #easm_lat_limits #euro_lat_limits #asia_lat_limits #nam_lat_limits

forest_frac_gc_nam_rec <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_nam_sim <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_nam_rec_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_nam_sim_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_nam_rec_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_nam_sim_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
nrec <- forest_frac_gc_nam_rec_mean <- forest_frac_gc_nam_sim_mean <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1))

for (t in 1:length(time)) {
  ind_tmp <- which(total$KaBP == time[t] & total$Longitude>=lon_limits[1] & total$Longitude<=lon_limits[2] & total$Latitude>=lat_limits[1] & total$Latitude<=lat_limits[2])
  rec <- total[ind_tmp,]
  for (k in 1:nr_samples) {
    ind_resample <- resample(1:nrow(rec),size=nrow(rec),replace=TRUE)
    rec_resample <- rec[ind_resample,]
    for (i in 1:(length(lon_seq)-1)) {
      for (j in 1:(length(lat_seq)-1)) {
        ind_tmp_local <- which(rec_resample$Longitude>=lon_seq[i] & rec_resample$Longitude<lon_seq[i+1] & rec_resample$Latitude>=lat_seq[j] & rec_resample$Latitude<lat_seq[j+1])
        if (length(ind_tmp_local)>=1) {
          forest_frac_gc_nam_rec[t,i,j,k] <- mean(rec_resample$biomREC[ind_tmp_local])
          forest_frac_gc_nam_sim[t,i,j,k] <- mean(rec_resample$Mega_biome[ind_tmp_local])
        }
      }
    }  
    forest_frac_gc_nam_rec_agg[t,k] <- mean(forest_frac_gc_nam_rec[t,,,k],na.rm=TRUE)
    forest_frac_gc_nam_sim_agg[t,k] <- mean(forest_frac_gc_nam_sim[t,,,k],na.rm=TRUE)
    forest_frac_gc_nam_rec_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_nam_rec[t,,,k])
    forest_frac_gc_nam_sim_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_nam_sim[t,,,k])
  }
}
##############
# Asia
#######################
lon_seq <- seq(-180,180,by=10)
lat_seq <- seq(30,80,by=5)
time <- sort(unique(total$KaBP))
nr_samples <- 1000
lon_limits <- asia_lon_limits #easm_lon_limits #euro_lon_limits #asia_lon_limits #nam_lon_limits
lat_limits <- asia_lat_limits #easm_lat_limits #euro_lat_limits #asia_lat_limits #nam_lat_limits

forest_frac_gc_as_rec <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_as_sim <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1,nr_samples))
forest_frac_gc_as_rec_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_as_sim_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_as_rec_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
forest_frac_gc_as_sim_spatial_agg <- array(NA,dim=c(length(time),nr_samples))
nrec <- forest_frac_gc_as_rec_mean <- forest_frac_gc_as_sim_mean <- array(NA,dim=c(length(time),length(lon_seq)-1,length(lat_seq)-1))

for (t in 1:length(time)) {
  ind_tmp <- which(total$KaBP == time[t] & total$Longitude>=lon_limits[1] & total$Longitude<=lon_limits[2] & total$Latitude>=lat_limits[1] & total$Latitude<=lat_limits[2])
  rec <- total[ind_tmp,]
  for (k in 1:nr_samples) {
    ind_resample <- resample(1:nrow(rec),size=nrow(rec),replace=TRUE)
    rec_resample <- rec[ind_resample,]
    for (i in 1:(length(lon_seq)-1)) {
      for (j in 1:(length(lat_seq)-1)) {
        ind_tmp_local <- which(rec_resample$Longitude>=lon_seq[i] & rec_resample$Longitude<lon_seq[i+1] & rec_resample$Latitude>=lat_seq[j] & rec_resample$Latitude<lat_seq[j+1])
        if (length(ind_tmp_local)>=1) {
          forest_frac_gc_as_rec[t,i,j,k] <- mean(rec_resample$biomREC[ind_tmp_local])
          forest_frac_gc_as_sim[t,i,j,k] <- mean(rec_resample$Mega_biome[ind_tmp_local])
        }
      }
    }  
    forest_frac_gc_as_rec_agg[t,k] <- mean(forest_frac_gc_as_rec[t,,,k],na.rm=TRUE)
    forest_frac_gc_as_sim_agg[t,k] <- mean(forest_frac_gc_as_sim[t,,,k],na.rm=TRUE)
    forest_frac_gc_as_rec_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_as_rec[t,,,k])
    forest_frac_gc_as_sim_spatial_agg[t,k] <- spatial_means(lon=c(lon_seq[-1]+lon_seq[-length(lon_seq)])/2,lat=c(lat_seq[-1]+lat_seq[-length(lat_seq)])/2,clim_field=forest_frac_gc_as_sim[t,,,k])
  }
}




###########################################################################

pdf("25krun_forest_sitebasedContmean_allsens.pdf")
par(mfrow=c(4,1))
par(mar = c(2, 5, 1, 15))


plot(-time*1000,apply(forest_frac_gc_as_rec_agg,1,mean),type="o",pch=20,col="white",ylim=c(0,1),yaxt="n",ylab="forest fraction", xaxt="n", xlab="",main="Asia")
ci_sim <- t(apply(forest_frac_gc_as_sim_agg,1,quantile,c(0.05,0.95)))
ci_rec <- t(apply(forest_frac_gc_as_rec_agg,1,quantile,c(0.05,0.95)))

ytick<-seq(0.0, 1.0, by=0.2)
xtick<-seq(-25000,0,5000)
axis(side=2, at=ytick, las=1, cex.axis=1.4)
#axis(side=1, at=xtick, las=1, cex.axis=1.3)

abline(h=seq(0,1,0.2), lty=3, col="darkgrey")
abline(v=seq(-25000,0,5000), lty=3, col="darkgrey")

#rect(-23000,-30,-19000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
#rect(-14690,-30,-12890,60, col= rgb(0.7,0,0,alpha=0.1), border = "transparent")
#rect(-12900,-30,-11600,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")

rect(-23000,-30,-18000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
rect(-10000,-30,0,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")


polygon(-c(time*1000,rev(time*1000)),c(ci_sim[,1],rev(ci_sim[,2])),col=adjustcolor("black",0.3),border="transparent")
polygon(-c(time*1000,rev(time*1000)),c(ci_rec[,1],rev(ci_rec[,2])),col=adjustcolor("red",0.3),border="transparent")
lines(-time*1000,apply(forest_frac_gc_as_rec_agg,1,mean),type="o",pch=20,col="red")
lines(-time*1000,apply(forest_frac_gc_as_sim_agg,1,mean),type="o",pch=20)


#####
plot(-time*1000,apply(forest_frac_gc_eur_rec_agg,1,mean),type="o",pch=20,col="white",ylim=c(0,1),yaxt="n",ylab="forest fraction", xaxt="n", xlab="",main="Europe")
ci_sim <- t(apply(forest_frac_gc_eur_sim_agg,1,quantile,c(0.05,0.95)))
ci_rec <- t(apply(forest_frac_gc_eur_rec_agg,1,quantile,c(0.05,0.95)))

ytick<-seq(0.0, 1.0, by=0.2)
xtick<-seq(-25000,0,5000)
axis(side=2, at=ytick, las=1, cex.axis=1.4)
#axis(side=1, at=xtick, las=1, cex.axis=1.3)

abline(h=seq(0,1,0.2), lty=3, col="darkgrey")
abline(v=seq(-25000,0,5000), lty=3, col="darkgrey")

#rect(-23000,-30,-19000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
#rect(-14690,-30,-12890,60, col= rgb(0.7,0,0,alpha=0.1), border = "transparent")
#rect(-12900,-30,-11600,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")

rect(-23000,-30,-18000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
rect(-10000,-30,0,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")


polygon(-c(time*1000,rev(time*1000)),c(ci_sim[,1],rev(ci_sim[,2])),col=adjustcolor("black",0.3),border="transparent")
polygon(-c(time*1000,rev(time*1000)),c(ci_rec[,1],rev(ci_rec[,2])),col=adjustcolor("red",0.3),border="transparent")
lines(-time*1000,apply(forest_frac_gc_eur_rec_agg,1,mean),type="o",pch=20,col="red")
lines(-time*1000,apply(forest_frac_gc_eur_sim_agg,1,mean),type="o",pch=20)


####################

plot(-time*1000,apply(forest_frac_gc_nam_rec_agg,1,mean),type="o",pch=20,col="white",ylim=c(0,1),yaxt="n",ylab="forest fraction", xaxt="n", xlab="years BP",main="NAmerica")
ci_sim <- t(apply(forest_frac_gc_nam_sim_agg,1,quantile,c(0.05,0.95)))
ci_rec <- t(apply(forest_frac_gc_nam_rec_agg,1,quantile,c(0.05,0.95)))

ytick<-seq(0.0, 1.0, by=0.2)
xtick<-seq(-25000,0,5000)
axis(side=2, at=ytick, las=1, cex.axis=1.4)
axis(side=1, at=xtick, las=1, cex.axis=1.4)

abline(h=seq(0,1,0.2), lty=3, col="darkgrey")
abline(v=seq(-25000,0,5000), lty=3, col="darkgrey")

#rect(-23000,-30,-19000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
#rect(-14690,-30,-12890,60, col= rgb(0.7,0,0,alpha=0.1), border = "transparent")
#rect(-12900,-30,-11600,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")

rect(-23000,-30,-18000,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")
rect(-10000,-30,0,60, col= rgb(0,0,0.7,alpha=0.1), border = "transparent")


polygon(-c(time*1000,rev(time*1000)),c(ci_sim[,1],rev(ci_sim[,2])),col=adjustcolor("black",0.3),border="transparent")
polygon(-c(time*1000,rev(time*1000)),c(ci_rec[,1],rev(ci_rec[,2])),col=adjustcolor("red",0.3),border="transparent")
lines(-time*1000,apply(forest_frac_gc_nam_rec_agg,1,mean),type="o",pch=20,col="red")
lines(-time*1000,apply(forest_frac_gc_nam_sim_agg,1,mean),type="o",pch=20)

legend("bottomright", inset=.055, legend=c("MPI-ESM", "Records"), col=c("black", "red"), lwd=3, lty=c(1,1),  xpd=NA, bg='white',y.intersp=0.92,x.intersp=0.92)

#######################

dev.off()

